@@ -1,8 +1,9 @@
 #! /usr/bin/env python
-import numpy
+import numpy as np
 import numpy.linalg as la
 import sys
 from pycuda.tools import mark_cuda_test
+from pycuda.characterize import has_double_support
 
 
 
@@ -22,43 +23,36 @@ if have_pycuda():
 
 
 
-def has_double_support():
-    from pycuda.driver import Context
-    return Context.get_device().compute_capability >= (1,3)
-
-
-
-
 class TestGPUArray:
     disabled = not have_pycuda()
 
     @mark_cuda_test
     def test_pow_array(self):
-        a = numpy.array([1,2,3,4,5]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
 
         result = pow(a_gpu,a_gpu).get()
-        assert (numpy.abs(a**a - result) < 1e-3).all()
+        assert (np.abs(a**a - result) < 1e-3).all()
 
         result = (a_gpu**a_gpu).get()
-        assert (numpy.abs(pow(a, a) - result) < 1e-3).all()
+        assert (np.abs(pow(a, a) - result) < 1e-3).all()
 
 
 
 
     @mark_cuda_test
     def test_pow_number(self):
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
 
         result = pow(a_gpu, 2).get()
-        assert (numpy.abs(a**2 - result) < 1e-3).all()
+        assert (np.abs(a**2 - result) < 1e-3).all()
 
 
 
     @mark_cuda_test
     def test_abs(self):
-        a = -gpuarray.arange(111, dtype=numpy.float32)
+        a = -gpuarray.arange(111, dtype=np.float32)
         res = a.get()
 
         for i in range(111):
@@ -75,7 +69,7 @@ class TestGPUArray:
 
     @mark_cuda_test
     def test_len(self):
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_cpu = gpuarray.to_gpu(a)
         assert len(a_cpu) == 10
 
@@ -88,11 +82,11 @@ class TestGPUArray:
 
         for sz in [10, 50000]:
             for dtype, scalars in [
-                (numpy.float32, [2]),
-                (numpy.complex64, [2, 2j])
+                (np.float32, [2]),
+                (np.complex64, [2, 2j])
                 ]:
                 for scalar in scalars:
-                    a = numpy.arange(sz).astype(dtype)
+                    a = np.arange(sz).astype(dtype)
                     a_gpu = gpuarray.to_gpu(a)
                     a_doubled = (scalar * a_gpu).get()
 
@@ -102,7 +96,7 @@ class TestGPUArray:
     def test_multiply_array(self):
         """Test the multiplication of two arrays."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
 
         a_gpu = gpuarray.to_gpu(a)
         b_gpu = gpuarray.to_gpu(a)
@@ -118,7 +112,7 @@ class TestGPUArray:
     def test_addition_array(self):
         """Test the addition of two arrays."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
         a_added = (a_gpu+a_gpu).get()
 
@@ -131,7 +125,7 @@ class TestGPUArray:
     def test_iaddition_array(self):
         """Test the inplace addition of two arrays."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
         a_gpu += a_gpu
         a_added = a_gpu.get()
@@ -144,7 +138,7 @@ class TestGPUArray:
     def test_addition_scalar(self):
         """Test the addition of an array and a scalar."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
         a_added = (7+a_gpu).get()
 
@@ -156,7 +150,7 @@ class TestGPUArray:
     def test_iaddition_scalar(self):
         """Test the inplace addition of an array and a scalar."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
         a_gpu += 7
         a_added = a_gpu.get()
@@ -170,8 +164,8 @@ class TestGPUArray:
     def test_substract_array(self):
         """Test the substraction of two arrays."""
         #test data
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
-        b = numpy.array([10,20,30,40,50,60,70,80,90,100]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
+        b = np.array([10,20,30,40,50,60,70,80,90,100]).astype(np.float32)
 
         a_gpu = gpuarray.to_gpu(a)
         b_gpu = gpuarray.to_gpu(b)
@@ -190,7 +184,7 @@ class TestGPUArray:
         """Test the substraction of an array and a scalar."""
 
         #test data
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
 
         #convert a to a gpu object
         a_gpu = gpuarray.to_gpu(a)
@@ -208,7 +202,7 @@ class TestGPUArray:
     def test_divide_scalar(self):
         """Test the division of an array and a scalar."""
 
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
 
         result = (a_gpu/2).get()
@@ -225,17 +219,17 @@ class TestGPUArray:
         """Test the division of an array and a scalar. """
 
         #test data
-        a = numpy.array([10,20,30,40,50,60,70,80,90,100]).astype(numpy.float32)
-        b = numpy.array([10,10,10,10,10,10,10,10,10,10]).astype(numpy.float32)
+        a = np.array([10,20,30,40,50,60,70,80,90,100]).astype(np.float32)
+        b = np.array([10,10,10,10,10,10,10,10,10,10]).astype(np.float32)
 
         a_gpu = gpuarray.to_gpu(a)
         b_gpu = gpuarray.to_gpu(b)
 
         a_divide = (a_gpu/b_gpu).get()
-        assert (numpy.abs(a/b - a_divide) < 1e-3).all()
+        assert (np.abs(a/b - a_divide) < 1e-3).all()
 
         a_divide = (b_gpu/a_gpu).get()
-        assert (numpy.abs(b/a - a_divide) < 1e-3).all()
+        assert (np.abs(b/a - a_divide) < 1e-3).all()
 
 
 
@@ -245,9 +239,9 @@ class TestGPUArray:
         from pycuda.curandom import rand as curand
 
         if has_double_support():
-            dtypes = [numpy.float32, numpy.float64]
+            dtypes = [np.float32, np.float64]
         else:
-            dtypes = [numpy.float32]
+            dtypes = [np.float32]
 
         for dtype in dtypes:
             a = curand((10, 100), dtype=dtype).get()
@@ -256,14 +250,53 @@ class TestGPUArray:
             assert (a < 1).all()
 
 
+
+
+    @mark_cuda_test
+    def test_curand_wrappers(self):
+        from pycuda.curandom import get_curand_version
+        if get_curand_version() is None:
+            from pytest import skip
+            skip("curand not installed")
+
+
+        from pycuda.curandom import (
+                XORWOWRandomNumberGenerator,
+                Sobol32RandomNumberGenerator)
+
+        if has_double_support():
+            dtypes = [np.float32, np.float64]
+        else:
+            dtypes = [np.float32]
+
+        for gen_type in [
+                XORWOWRandomNumberGenerator,
+                #Sobol32RandomNumberGenerator
+                ]:
+            gen = gen_type()
+
+            for dtype in dtypes:
+                gen.gen_normal(10000, dtype)
+                # test non-Box-Muller version, if available
+                gen.gen_normal(10001, dtype)
+
+                x = gen.gen_uniform(10000, dtype)
+                x_host = x.get()
+                assert (-1 <= x_host).all()
+                assert (x_host <= 1).all()
+
+            gen.gen_uniform(10000, np.uint32)
+
+
+
     @mark_cuda_test
     def test_array_gt(self):
         """Test whether array contents are > the other array's
         contents"""
 
-        a = numpy.array([5,10]).astype(numpy.float32)
+        a = np.array([5,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10]).astype(numpy.float32)
+        b = np.array([2,10]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (a_gpu > b_gpu).get()
         assert result[0] == True
@@ -274,9 +307,9 @@ class TestGPUArray:
         """Test whether array contents are < the other array's
         contents"""
 
-        a = numpy.array([5,10]).astype(numpy.float32)
+        a = np.array([5,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10]).astype(numpy.float32)
+        b = np.array([2,10]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (b_gpu < a_gpu).get()
         assert result[0] == True
@@ -287,9 +320,9 @@ class TestGPUArray:
         """Test whether array contents are <= the other array's
         contents"""
 
-        a = numpy.array([5,10, 1]).astype(numpy.float32)
+        a = np.array([5,10, 1]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10, 2]).astype(numpy.float32)
+        b = np.array([2,10, 2]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (b_gpu <= a_gpu).get()
         assert result[0] == True
@@ -301,9 +334,9 @@ class TestGPUArray:
         """Test whether array contents are >= the other array's
         contents"""
 
-        a = numpy.array([5,10,1]).astype(numpy.float32)
+        a = np.array([5,10,1]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10,2]).astype(numpy.float32)
+        b = np.array([2,10,2]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (a_gpu >= b_gpu).get()
         assert result[0] == True
@@ -315,9 +348,9 @@ class TestGPUArray:
         """Test whether array contents are == the other array's
         contents"""
 
-        a = numpy.array([5,10]).astype(numpy.float32)
+        a = np.array([5,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10]).astype(numpy.float32)
+        b = np.array([2,10]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (a_gpu == b_gpu).get()
         assert result[0] == False
@@ -328,9 +361,9 @@ class TestGPUArray:
         """Test whether array contents are != the other array's
         contents"""
 
-        a = numpy.array([5,10]).astype(numpy.float32)
+        a = np.array([5,10]).astype(np.float32)
         a_gpu = gpuarray.to_gpu(a)
-        b = numpy.array([2,10]).astype(numpy.float32)
+        b = np.array([2,10]).astype(np.float32)
         b_gpu = gpuarray.to_gpu(b)
         result = (a_gpu != b_gpu).get()
         assert result[0] == True
@@ -341,7 +374,7 @@ class TestGPUArray:
     def test_nan_arithmetic(self):
         def make_nan_contaminated_vector(size):
             shape = (size,)
-            a = numpy.random.randn(*shape).astype(numpy.float32)
+            a = np.random.randn(*shape).astype(np.float32)
             #for i in range(0, shape[0], 3):
                 #a[i] = float('nan')
             from random import randrange
@@ -359,8 +392,7 @@ class TestGPUArray:
         ab = a*b
         ab_gpu = (a_gpu*b_gpu).get()
 
-        for i in range(size):
-            assert numpy.isnan(ab[i]) == numpy.isnan(ab_gpu[i])
+        assert (np.isnan(ab) == np.isnan(ab_gpu)).all()
 
 
 
@@ -386,10 +418,37 @@ class TestGPUArray:
 
 
 
+    @mark_cuda_test
+    def test_ranged_elwise_kernel(self):
+        from pycuda.elementwise import ElementwiseKernel
+        set_to_seven = ElementwiseKernel(
+                "float *z",
+                "z[i] = 7",
+                "set_to_seven")
+
+        for i, slc in enumerate([
+                slice(5, 20000),
+                slice(5, 20000, 17),
+                slice(3000, 5, -1),
+                slice(1000, -1),
+                ]):
+
+            a_gpu = gpuarray.zeros((50000,), dtype=np.float32)
+            a_cpu = np.zeros(a_gpu.shape, a_gpu.dtype)
+
+            a_cpu[slc] = 7
+            set_to_seven(a_gpu, slice=slc)
+            drv.Context.synchronize()
+
+            assert la.norm(a_cpu - a_gpu.get()) == 0, i
+
+
+
+
     @mark_cuda_test
     def test_take(self):
-        idx = gpuarray.arange(0, 200000, 2, dtype=numpy.uint32)
-        a = gpuarray.arange(0, 600000, 3, dtype=numpy.float32)
+        idx = gpuarray.arange(0, 200000, 2, dtype=np.uint32)
+        a = gpuarray.arange(0, 600000, 3, dtype=np.float32)
         result = gpuarray.take(a, idx)
         assert ((3*idx).get() == result.get()).all()
 
@@ -398,15 +457,15 @@ class TestGPUArray:
 
     @mark_cuda_test
     def test_arange(self):
-        a = gpuarray.arange(12, dtype=numpy.float32)
-        assert (numpy.arange(12, dtype=numpy.float32) == a.get()).all()
+        a = gpuarray.arange(12, dtype=np.float32)
+        assert (np.arange(12, dtype=np.float32) == a.get()).all()
 
 
 
 
     @mark_cuda_test
     def test_reverse(self):
-        a = numpy.array([1,2,3,4,5,6,7,8,9,10]).astype(numpy.float32)
+        a = np.array([1,2,3,4,5,6,7,8,9,10]).astype(np.float32)
         a_cpu = gpuarray.to_gpu(a)
 
         a_cpu = a_cpu.reverse()
@@ -423,7 +482,7 @@ class TestGPUArray:
         a_gpu = curand((200000,))
         a = a_gpu.get()
 
-        sum_a = numpy.sum(a)
+        sum_a = np.sum(a)
 
         from pycuda.reduction import get_sum_kernel
         sum_a_gpu = gpuarray.sum(a_gpu).get()
@@ -435,16 +494,16 @@ class TestGPUArray:
         from pycuda.curandom import rand as curand
 
         if has_double_support():
-            dtypes = [numpy.float64, numpy.float32, numpy.int32]
+            dtypes = [np.float64, np.float32, np.int32]
         else:
-            dtypes = [numpy.float32, numpy.int32]
+            dtypes = [np.float32, np.int32]
 
         for what in ["min", "max"]:
             for dtype in dtypes:
                 a_gpu = curand((200000,), dtype)
                 a = a_gpu.get()
 
-                op_a = getattr(numpy, what)(a)
+                op_a = getattr(np, what)(a)
                 op_a_gpu = getattr(gpuarray, what)(a_gpu).get()
 
                 assert op_a_gpu == op_a, (op_a_gpu, op_a, dtype, what)
@@ -458,15 +517,15 @@ class TestGPUArray:
         l_m = l_a - l_a // gran + 1
 
         if has_double_support():
-            dtypes = [numpy.float64, numpy.float32, numpy.int32]
+            dtypes = [np.float64, np.float32, np.int32]
         else:
-            dtypes = [numpy.float32, numpy.int32]
+            dtypes = [np.float32, np.int32]
 
         for dtype in dtypes:
             a_gpu = curand((l_a,), dtype)
             a = a_gpu.get()
 
-            meaningful_indices_gpu = gpuarray.zeros(l_m, dtype=numpy.int32)
+            meaningful_indices_gpu = gpuarray.zeros(l_m, dtype=np.int32)
             meaningful_indices = meaningful_indices_gpu.get()
             j = 0
             for i in range(len(meaningful_indices)):
@@ -478,7 +537,7 @@ class TestGPUArray:
             meaningful_indices_gpu = gpuarray.to_gpu(meaningful_indices)
             b = a[meaningful_indices]
 
-            min_a = numpy.min(b)
+            min_a = np.min(b)
             min_a_gpu = gpuarray.subset_min(meaningful_indices_gpu, a_gpu).get()
 
             assert min_a_gpu == min_a
@@ -491,7 +550,7 @@ class TestGPUArray:
         b_gpu = curand((200000,))
         b = b_gpu.get()
 
-        dot_ab = numpy.dot(a, b)
+        dot_ab = np.dot(a, b)
 
         dot_ab_gpu = gpuarray.dot(a_gpu, b_gpu).get()
 
@@ -531,18 +590,18 @@ class TestGPUArray:
         min_a_b_gpu = gpuarray.minimum(a_gpu, b_gpu)
 
         print max_a_b_gpu
-        print numpy.maximum(a, b)
+        print np.maximum(a, b)
 
-        assert la.norm(max_a_b_gpu.get()- numpy.maximum(a, b)) == 0
-        assert la.norm(min_a_b_gpu.get()- numpy.minimum(a, b)) == 0
+        assert la.norm(max_a_b_gpu.get()- np.maximum(a, b)) == 0
+        assert la.norm(min_a_b_gpu.get()- np.minimum(a, b)) == 0
 
     @mark_cuda_test
     def test_take_put(self):
         for n in [5, 17, 333]:
             one_field_size = 8
-            buf_gpu = gpuarray.zeros(n*one_field_size, dtype=numpy.float32)
-            dest_indices = gpuarray.to_gpu(numpy.array([ 0,  1,  2,  3, 32, 33, 34, 35], dtype=numpy.uint32))
-            read_map = gpuarray.to_gpu(numpy.array([7, 6, 5, 4, 3, 2, 1, 0], dtype=numpy.uint32))
+            buf_gpu = gpuarray.zeros(n*one_field_size, dtype=np.float32)
+            dest_indices = gpuarray.to_gpu(np.array([ 0,  1,  2,  3, 32, 33, 34, 35], dtype=np.uint32))
+            read_map = gpuarray.to_gpu(np.array([7, 6, 5, 4, 3, 2, 1, 0], dtype=np.uint32))
 
             gpuarray.multi_take_put(
                     arrays=[buf_gpu for i in range(n)],
@@ -560,20 +619,20 @@ class TestGPUArray:
         if not has_double_support():
             return
 
-        a_gpu = curand((2000,), dtype=numpy.float32)
+        a_gpu = curand((2000,), dtype=np.float32)
 
-        a = a_gpu.get().astype(numpy.float64)
-        a2 = a_gpu.astype(numpy.float64).get()
+        a = a_gpu.get().astype(np.float64)
+        a2 = a_gpu.astype(np.float64).get()
 
-        assert a2.dtype == numpy.float64
+        assert a2.dtype == np.float64
         assert la.norm(a - a2) == 0, (a, a2)
 
-        a_gpu = curand((2000,), dtype=numpy.float64)
+        a_gpu = curand((2000,), dtype=np.float64)
 
-        a = a_gpu.get().astype(numpy.float32)
-        a2 = a_gpu.astype(numpy.float32).get()
+        a = a_gpu.get().astype(np.float32)
+        a2 = a_gpu.astype(np.float32).get()
 
-        assert a2.dtype == numpy.float32
+        assert a2.dtype == np.float32
         assert la.norm(a - a2)/la.norm(a) < 1e-7
 
     @mark_cuda_test
@@ -581,15 +640,15 @@ class TestGPUArray:
         from pycuda.curandom import rand as curand
 
         if has_double_support():
-            dtypes = [numpy.complex64, numpy.complex128]
+            dtypes = [np.complex64, np.complex128]
         else:
-            dtypes = [numpy.complex64]
+            dtypes = [np.complex64]
 
         n = 20
         for tp in dtypes:
-            dtype = numpy.dtype(tp)
+            dtype = np.dtype(tp)
             from pytools import match_precision
-            real_dtype = match_precision(numpy.dtype(numpy.float64), dtype)
+            real_dtype = match_precision(np.dtype(np.float64), dtype)
 
             z = (curand((n,), real_dtype).astype(dtype)
                     + 1j*curand((n,), real_dtype).astype(dtype))
@@ -598,6 +657,71 @@ class TestGPUArray:
             assert la.norm(z.get().imag - z.imag.get()) == 0
             assert la.norm(z.get().conj() - z.conj().get()) == 0
 
+    @mark_cuda_test
+    def test_pass_slice_to_kernel(self):
+        mod = SourceModule("""
+        __global__ void twice(float *a)
+        {
+          const int i = threadIdx.x + blockIdx.x * blockDim.x;
+          a[i] *= 2;
+        }
+        """)
+
+        multiply_them = mod.get_function("twice")
+
+        a = np.ones(256**2, np.float32)
+        a_gpu = gpuarray.to_gpu(a)
+
+        multiply_them(a_gpu[256:-256], block=(256,1,1), grid=(254,1))
+
+        a = a_gpu.get()
+        assert (a[255:257]== np.array([1,2], np.float32)).all()
+        assert (a[255*256-1:255*256+1] == np.array([2,1], np.float32)).all()
+
+    @mark_cuda_test
+    def test_scan(self):
+        from pycuda.scan import ExclusiveScanKernel, InclusiveScanKernel
+        for cls in [ExclusiveScanKernel, InclusiveScanKernel]:
+            scan_kern = cls(np.int32, "a+b", "0")
+
+            for n in [
+                    10, 2**10-5, 2**10, 
+                    2**20-2**18, 
+                    2**20-2**18+5, 
+                    2**10+5,
+                    2**20+5,
+                    2**20, 2**24
+                    ]:
+                host_data = np.random.randint(0, 10, n).astype(np.int32)
+                gpu_data = gpuarray.to_gpu(host_data)
+
+                scan_kern(gpu_data)
+
+                desired_result = np.cumsum(host_data, axis=0)
+                if cls is ExclusiveScanKernel:
+                    desired_result -= host_data
+
+                assert (gpu_data.get() == desired_result).all()
+
+    @mark_cuda_test
+    def test_stride_preservation(self):
+        A = np.random.rand(3,3)
+        AT = A.T
+        print AT.flags.f_contiguous, AT.flags.c_contiguous
+        AT_GPU = gpuarray.to_gpu(AT)
+        print AT_GPU.flags.f_contiguous, AT_GPU.flags.c_contiguous
+        assert np.allclose(AT_GPU.get(),AT)
+
+    @mark_cuda_test
+    def test_vector_fill(self):
+        a_gpu = gpuarray.GPUArray(100, dtype=gpuarray.vec.float3)
+        a_gpu.fill(gpuarray.vec.make_float3(0.0, 0.0, 0.0))
+        a = a_gpu.get()
+        assert a.dtype is gpuarray.vec.float3
+
+
+
+
 
 if __name__ == "__main__":
     # make sure that import failures get reported, instead of skipping the tests.
